"""
The yaaredis integration traces yaaredis requests.


Enabling
~~~~~~~~

The yaaredis integration is enabled automatically when using
:ref:`ddtrace-run<ddtracerun>` or :func:`patch_all()<ddtrace.patch_all>`.

Or use :func:`patch()<ddtrace.patch>` to manually enable the integration::

    from ddtrace import patch
    patch(yaaredis=True)


Global Configuration
~~~~~~~~~~~~~~~~~~~~

.. py:data:: ddtrace.config.yaaredis["service"]

   The service name reported by default for yaaredis traces.

   This option can also be set with the ``DD_YAAREDIS_SERVICE`` environment
   variable.

   Default: ``"redis"``

.. py:data:: ddtrace.config.yaaredis["cmd_max_length"]

   Max allowable size for the yaaredis command span tag.
   Anything beyond the max length will be replaced with ``"..."``.

   This option can also be set with the ``DD_YAAREDIS_CMD_MAX_LENGTH`` environment
   variable.

   Default: ``1000``


Instance Configuration
~~~~~~~~~~~~~~~~~~~~~~

To configure particular yaaredis instances use the :class:`Pin <ddtrace.Pin>` API::

    import yaaredis
    from ddtrace import Pin

    client = yaaredis.StrictRedis(host="localhost", port=6379)

    # Override service name for this instance
    Pin.override(client, service="my-custom-queue")

    # Traces reported for this client will now have "my-custom-queue"
    # as the service name.
    async def example():
        await client.get("my-key")
"""

from ...internal.utils.importlib import require_modules


required_modules = ["yaaredis", "yaaredis.client"]

with require_modules(required_modules) as missing_modules:
    if not missing_modules:
        from .patch import patch

        __all__ = ["patch"]
